@@ -104,7 +104,10 @@
 #ifndef TOTEM_PL_PARSER_MINI
 #include <gobject/gvaluecollector.h>
 #include <gtk/gtk.h>
+
+#ifdef HAVE_CAMEL
 #include <camel/camel-mime-utils.h>
+#endif
 
 #include "totem-pl-parser.h"
 #include "totemplparser-marshal.h"
@@ -132,7 +135,7 @@
 typedef const char * (*PlaylistIdenCallback) (const char *data, gsize len);
 
 #ifndef TOTEM_PL_PARSER_MINI
-typedef TotemPlParserResult (*PlaylistCallback) (TotemPlParser *parser, GFile *url, GFile *base_file, gpointer data);
+typedef TotemPlParserResult (*PlaylistCallback) (TotemPlParser *parser, GFile *uri, GFile *base_file, gpointer data);
 #endif
 
 typedef struct {
@@ -199,6 +202,7 @@ static PlaylistTypes dual_types[] = {
 	PLAYLIST_TYPE2 ("video/x-ms-asf", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
 	PLAYLIST_TYPE2 ("video/x-ms-wmv", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
 	PLAYLIST_TYPE2 ("video/quicktime", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
+	PLAYLIST_TYPE2 ("video/mp4", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
 	PLAYLIST_TYPE2 ("application/x-quicktime-media-link", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
 	PLAYLIST_TYPE2 ("application/x-quicktimeplayer", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
 	PLAYLIST_TYPE2 ("application/xml", totem_pl_parser_add_xml_feed, totem_pl_parser_is_xml_feed),
@@ -294,7 +298,7 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 					 PROP_RECURSE,
 					 g_param_spec_boolean ("recurse",
 							       "recurse",
-							       "Whether or not to process URLs further", 
+							       "Whether or not to process URIs further", 
 							       TRUE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
@@ -355,7 +359,7 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 			      G_STRUCT_OFFSET (TotemPlParserClass, entry_parsed),
 			      NULL, NULL,
 			      totemplparser_marshal_VOID__STRING_BOXED,
-			      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_HASH_TABLE);
+			      G_TYPE_NONE, 2, G_TYPE_STRING, TOTEM_TYPE_PL_PARSER_METADATA);
 	/**
 	 * TotemPlParser::playlist-started:
 	 * @parser: the object which received the signal
@@ -375,7 +379,7 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 			      G_STRUCT_OFFSET (TotemPlParserClass, playlist_started),
 			      NULL, NULL,
 			      totemplparser_marshal_VOID__STRING_BOXED,
-			      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_HASH_TABLE);
+			      G_TYPE_NONE, 2, G_TYPE_STRING, TOTEM_TYPE_PL_PARSER_METADATA);
 	/**
 	 * TotemPlParser::playlist-ended:
 	 * @parser: the object which received the signal
@@ -397,7 +401,7 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 	/* param specs */
 	totem_pl_parser_pspec_pool = g_param_spec_pool_new (FALSE);
 	pspec = g_param_spec_string ("url", "url",
-				     "URL to be added", NULL,
+				     "URI to be added", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 	pspec = g_param_spec_string ("title", "title",
@@ -417,7 +421,7 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 	pspec = g_param_spec_string ("base", "base",
-				     "Base URL of the item to be added", NULL,
+				     "Base URI of the item to be added", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 	pspec = g_param_spec_string ("volume", "volume",
@@ -449,7 +453,7 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 	pspec = g_param_spec_string ("moreinfo", "moreinfo",
-				     "URL to get more information for item to be added", NULL,
+				     "URI to get more information for item to be added", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 	pspec = g_param_spec_string ("screensize", "screensize",
@@ -496,8 +500,12 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 				     "Object representing the GFile for an entry", G_TYPE_FILE,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_object ("gfile-object-base", "gfile-object-base",
+				     "Object representing the GFile for base URI of an entry", G_TYPE_FILE,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 	pspec = g_param_spec_string ("download-url", "download-url",
-				     "String representing the location of a download URL", NULL,
+				     "String representing the location of a download URI", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 	pspec = g_param_spec_string ("id", "id",
@@ -659,17 +667,18 @@ my_g_file_info_get_mime_type_with_data (GFile *file, gpointer *data, TotemPlPars
 			g_error_free (error);
 			return g_strdup (DIR_MIME_TYPE);
 		}
-		DEBUG(file, g_print ("URL '%s' couldn't be opened in _get_mime_type_with_data: '%s'\n", uri, error->message));
+		DEBUG(file, g_print ("URI '%s' couldn't be opened in _get_mime_type_with_data: '%s'\n", uri, error->message));
 		g_error_free (error);
 		return NULL;
 	}
-	DEBUG(file, g_print ("URL '%s' was opened successfully in _get_mime_type_with_data:\n", uri));
+	DEBUG(file, g_print ("URI '%s' was opened successfully in _get_mime_type_with_data\n", uri));
 
 	/* Read the whole thing, up to MIME_READ_CHUNK_SIZE */
 	buffer = g_malloc (MIME_READ_CHUNK_SIZE);
 	bytes_read = g_input_stream_read (G_INPUT_STREAM (stream), buffer, MIME_READ_CHUNK_SIZE, NULL, &error);
-	g_input_stream_close (G_INPUT_STREAM (stream), NULL, NULL);
+	g_object_unref (G_INPUT_STREAM (stream));
 	if (bytes_read == -1) {
+		DEBUG(file, g_print ("Couldn't read data from '%s'\n", uri));
 		g_free (buffer);
 		return NULL;
 	}
@@ -677,7 +686,7 @@ my_g_file_info_get_mime_type_with_data (GFile *file, gpointer *data, TotemPlPars
 	/* Empty file */
 	if (bytes_read == 0) {
 		g_free (buffer);
-		DEBUG(file, g_print ("URL '%s' is empty in _get_mime_type_with_data\n", uri));
+		DEBUG(file, g_print ("URI '%s' is empty in _get_mime_type_with_data\n", uri));
 		return g_strdup (EMPTY_FILE_TYPE);
 	}
 
@@ -690,22 +699,22 @@ my_g_file_info_get_mime_type_with_data (GFile *file, gpointer *data, TotemPlPars
 }
 
 /**
- * totem_pl_parser_base_url:
- * @url: a URI
+ * totem_pl_parser_base_uri:
+ * @uri: a URI
  *
- * Returns the parent URI of @url.
+ * Returns the parent URI of @uri.
  *
- * Return value: a newly-allocated string containing @url's parent URI, or %NULL
+ * Return value: a newly-allocated string containing @uri's parent URI, or %NULL
  **/
 char *
-totem_pl_parser_base_url (GFile *file)
+totem_pl_parser_base_uri (GFile *uri)
 {
 	GFile *parent;
 	char *ret;
 
-	parent = g_file_get_parent (file);
+	parent = g_file_get_parent (uri);
 	ret = g_file_get_uri (parent);
-	g_object_unref (file);
+	g_object_unref (uri);
 
 	return ret;
 }
@@ -741,6 +750,7 @@ totem_pl_parser_line_is_empty (const char *line)
  * @error: return location for a #GError, or %NULL
  *
  * Writes the string @buf out to the file specified by @handle.
+ * Possible error codes are as per totem_pl_parser_write_buffer().
  *
  * Return value: %TRUE on success
  **/
@@ -762,6 +772,8 @@ totem_pl_parser_write_string (GOutputStream *stream, const char *buf, GError **e
  *
  * Writes @len bytes of @buf to the file specified by @handle.
  *
+ * A value of @len greater than #G_MAXSSIZE will cause a #G_IO_ERROR_INVALID_ARGUMENT argument.
+ *
  * Return value: %TRUE on success
  **/
 gboolean
@@ -773,7 +785,7 @@ totem_pl_parser_write_buffer (GOutputStream *stream, const char *buf, guint len,
 				       buf, len,
 				       &bytes_written,
 				       NULL, error) == FALSE) {
-		g_output_stream_close (stream, NULL, NULL);
+		g_object_unref (stream);
 		return FALSE;
 	}
 
@@ -804,20 +816,20 @@ totem_pl_parser_num_entries (TotemPlParser *parser, GtkTreeModel *model,
 	for (i = 1; i <= num_entries; i++)
 	{
 		GtkTreeIter iter;
-		char *url, *title;
+		char *uri, *title;
 		GFile *file;
 		gboolean custom_title;
 
 		if (gtk_tree_model_iter_nth_child (model, &iter, NULL, i - 1) == FALSE)
 			return i - ignored;
 
-		func (model, &iter, &url, &title, &custom_title, user_data);
-		file = g_file_new_for_uri (url);
+		func (model, &iter, &uri, &title, &custom_title, user_data);
+		file = g_file_new_for_uri (uri);
 		if (totem_pl_parser_scheme_is_ignored (parser, file) != FALSE)
 			ignored++;
 
 		g_object_unref (file);
-		g_free (url);
+		g_free (uri);
 		g_free (title);
 	}
 
@@ -861,6 +873,20 @@ totem_pl_parser_relative (GFile *output, const char *filepath)
  * @user_data), which gets various metadata values about the entry for
  * the playlist writer.
  *
+ * If the @output file is a directory the #G_IO_ERROR_IS_DIRECTORY error
+ * will be returned, and if the file is some other form of non-regular file
+ * then a #G_IO_ERROR_NOT_REGULAR_FILE error will be returned. Some file
+ * systems don't allow all file names, and may return a
+ * #G_IO_ERROR_INVALID_FILENAME error, and if the name is too long,
+ * #G_IO_ERROR_FILENAME_TOO_LONG will be returned. Other errors are possible
+ * too, and depend on what kind of filesystem the file is on.
+ *
+ * In extreme cases, a #G_IO_ERROR_INVALID_ARGUMENT error can be returned, if
+ * parts of the playlist to be written are too long.
+ *
+ * If writing a PLA playlist and there is an error converting a URI's encoding,
+ * a code from #GConvertError will be returned.
+ *
  * Return value: %TRUE on success
  **/
 gboolean
@@ -917,6 +943,8 @@ totem_pl_parser_write_with_title (TotemPlParser *parser, GtkTreeModel *model,
  * @user_data), which gets various metadata values about the entry for
  * the playlist writer.
  *
+ * Possible error codes are as per totem_pl_parser_write_with_title().
+ *
  * Return value: %TRUE on success
  **/
 gboolean
@@ -1068,16 +1096,16 @@ totem_pl_parser_finalize (GObject *object)
 }
 
 static void
-totem_pl_parser_add_url_valist (TotemPlParser *parser,
+totem_pl_parser_add_uri_valist (TotemPlParser *parser,
 				const gchar *first_property_name,
 				va_list      var_args)
 {
 	const char *name;
-	char *title, *url;
+	char *title, *uri;
 	GHashTable *metadata;
 	gboolean is_playlist;
 
-	title = url = NULL;
+	title = uri = NULL;
 	is_playlist = FALSE;
 
 	g_object_ref (G_OBJECT (parser));
@@ -1109,28 +1137,28 @@ totem_pl_parser_add_url_valist (TotemPlParser *parser,
 			break;
 		}
 
-		if (strcmp (name, TOTEM_PL_PARSER_FIELD_URL) == 0) {
-			if (url == NULL)
-				url = g_value_dup_string (&value);
+		if (strcmp (name, TOTEM_PL_PARSER_FIELD_URI) == 0) {
+			if (uri == NULL)
+				uri = g_value_dup_string (&value);
 		} else if (strcmp (name, TOTEM_PL_PARSER_FIELD_FILE) == 0) {
 			GFile *file;
 
 			file = g_value_get_object (&value);
-			url = g_file_get_uri (file);
+			uri = g_file_get_uri (file);
 
 			g_value_unset (&value);
 			name = va_arg (var_args, char*);
 			continue;
 		} else if (strcmp (name, TOTEM_PL_PARSER_FIELD_BASE_FILE) == 0) {
 			GFile *file;
-			char *base_url;
+			char *base_uri;
 
 			file = g_value_get_object (&value);
-			base_url = g_file_get_uri (file);
+			base_uri = g_file_get_uri (file);
 
 			g_hash_table_insert (metadata,
 					     g_strdup (TOTEM_PL_PARSER_FIELD_BASE),
-					     base_url);
+					     base_uri);
 
 			g_value_unset (&value);
 			name = va_arg (var_args, char*);
@@ -1178,58 +1206,58 @@ totem_pl_parser_add_url_valist (TotemPlParser *parser,
 		//FIXME fix this! 396710
 	}
 
-	if (g_hash_table_size (metadata) > 0 || url != NULL) {
+	if (g_hash_table_size (metadata) > 0 || uri != NULL) {
 		if (is_playlist == FALSE) {
 			g_signal_emit (G_OBJECT (parser),
 				       totem_pl_parser_table_signals[ENTRY_PARSED],
-				       0, url, metadata);
+				       0, uri, metadata);
 		} else {
 			g_signal_emit (G_OBJECT (parser),
 				       totem_pl_parser_table_signals[PLAYLIST_STARTED],
-				       0, url, metadata);
+				       0, uri, metadata);
 		}
 	}
 
 	g_hash_table_unref (metadata);
 
-	g_free (url);
+	g_free (uri);
 	g_object_unref (G_OBJECT (parser));
 }
 
 /**
- * totem_pl_parser_add_url:
+ * totem_pl_parser_add_uri:
  * @parser: a #TotemPlParser
  * @first_property_name: the first property name
  * @Varargs: value for the first property, followed optionally by more
  * name/value pairs, followed by %NULL
  *
- * Adds a URL to the playlist with the properties given in @first_property_name
+ * Adds a URI to the playlist with the properties given in @first_property_name
  * and @Varargs.
  **/
 void
-totem_pl_parser_add_url (TotemPlParser *parser,
+totem_pl_parser_add_uri (TotemPlParser *parser,
 			 const char *first_property_name,
 			 ...)
 {
 	va_list var_args;
 	va_start (var_args, first_property_name);
-	totem_pl_parser_add_url_valist (parser, first_property_name, var_args);
+	totem_pl_parser_add_uri_valist (parser, first_property_name, var_args);
 	va_end (var_args);
 }
 
 /**
- * totem_pl_parser_add_one_url:
+ * totem_pl_parser_add_one_uri:
  * @parser: a #TotemPlParser
- * @url: the entry's URL
+ * @uri: the entry's URI
  * @title: the entry's title
  *
- * Adds a single URL entry with only URL and title strings to the playlist.
+ * Adds a single URI entry with only URI and title strings to the playlist.
  **/
 void
-totem_pl_parser_add_one_url (TotemPlParser *parser, const char *url, const char *title)
+totem_pl_parser_add_one_uri (TotemPlParser *parser, const char *uri, const char *title)
 {
-	totem_pl_parser_add_url (parser,
-				 TOTEM_PL_PARSER_FIELD_URL, url,
+	totem_pl_parser_add_uri (parser,
+				 TOTEM_PL_PARSER_FIELD_URI, uri,
 				 TOTEM_PL_PARSER_FIELD_TITLE, title,
 				 NULL);
 }
@@ -1237,7 +1265,7 @@ totem_pl_parser_add_one_url (TotemPlParser *parser, const char *url, const char
 void
 totem_pl_parser_add_one_file (TotemPlParser *parser, GFile *file, const char *title)
 {
-	totem_pl_parser_add_url (parser,
+	totem_pl_parser_add_uri (parser,
 				 TOTEM_PL_PARSER_FIELD_FILE, file,
 				 TOTEM_PL_PARSER_FIELD_TITLE, title,
 				 NULL);
@@ -1254,15 +1282,15 @@ static PlaylistTypes ignore_types[] = {
 /**
  * totem_pl_parser_scheme_is_ignored:
  * @parser: a #TotemPlParser
- * @url: a URL
+ * @uri: a URI
  *
- * Checks to see if @url's scheme is in the @parser's list of
+ * Checks to see if @uri's scheme is in the @parser's list of
  * schemes to ignore.
  *
- * Return value: %TRUE if @url's scheme is ignored
+ * Return value: %TRUE if @uri's scheme is ignored
  **/
 gboolean
-totem_pl_parser_scheme_is_ignored (TotemPlParser *parser, GFile *file)
+totem_pl_parser_scheme_is_ignored (TotemPlParser *parser, GFile *uri)
 {
 	GList *l;
 
@@ -1271,7 +1299,7 @@ totem_pl_parser_scheme_is_ignored (TotemPlParser *parser, GFile *file)
 
 	for (l = parser->priv->ignore_schemes; l != NULL; l = l->next) {
 		const char *scheme = l->data;
-		if (g_file_has_uri_scheme (file, scheme) != FALSE)
+		if (g_file_has_uri_scheme (uri, scheme) != FALSE)
 			return TRUE;
 	}
 
@@ -1300,28 +1328,28 @@ totem_pl_parser_mimetype_is_ignored (TotemPlParser *parser,
 /**
  * totem_pl_parser_ignore:
  * @parser: a #TotemPlParser
- * @url: a URL
+ * @uri: a URI
  *
- * Checks if the URL should be ignored. URLs are <emphasis>not</emphasis> ignored if:
+ * Checks if the URI should be ignored. URIs are <emphasis>not</emphasis> ignored if:
  * <itemizedlist>
  *  <listitem><para>they have an unknown mimetype,</para></listitem>
  *  <listitem><para>they have a special mimetype,</para></listitem>
  *  <listitem><para>they have a mimetype which could be a video or a playlist.</para></listitem>
  * </itemizedlist>
  *
- * URLs are automatically ignored if their scheme is ignored as per totem_pl_parser_scheme_is_ignored(),
+ * URIs are automatically ignored if their scheme is ignored as per totem_pl_parser_scheme_is_ignored(),
  * and are ignored if all the other tests are inconclusive.
  *
- * Return value: %TRUE if @url is to be ignored
+ * Return value: %TRUE if @uri is to be ignored
  **/
 gboolean
-totem_pl_parser_ignore (TotemPlParser *parser, const char *url)
+totem_pl_parser_ignore (TotemPlParser *parser, const char *uri)
 {
 	char *mimetype;
 	GFile *file;
 	guint i;
 
-	file = g_file_new_for_path (url);
+	file = g_file_new_for_path (uri);
 	if (totem_pl_parser_scheme_is_ignored (parser, file) != FALSE) {
 		g_object_unref (file);
 		return TRUE;
@@ -1329,7 +1357,7 @@ totem_pl_parser_ignore (TotemPlParser *parser, const char *url)
 	g_object_unref (file);
 
 	//FIXME wrong for win32
-	mimetype = g_content_type_guess (url, NULL, 0, NULL);
+	mimetype = g_content_type_guess (uri, NULL, 0, NULL);
 	if (mimetype == NULL || strcmp (mimetype, UNKNOWN_TYPE) == 0) {
 		g_free (mimetype);
 		return FALSE;
@@ -1354,6 +1382,33 @@ totem_pl_parser_ignore (TotemPlParser *parser, const char *url)
 	return TRUE;
 }
 
+/**
+ * totem_pl_parser_cleanup_xml:
+ * @contents: the contents of the file
+ *
+ * Removes HTML comments from a string representing the contents of an XML file.
+ * The function modifies the string in place.
+ */
+void
+totem_pl_parser_cleanup_xml (char *contents)
+{
+	char *needle;
+
+	while ((needle = strstr (contents, "<!--")) != NULL) {
+		while (strncmp (needle, "-->", 3) != 0) {
+			*needle = ' ';
+			needle++;
+			if (*needle == '\0')
+				break;
+		}
+		if (strncmp (needle, "-->", 3) == 0) {
+			guint i;
+			for (i = 0; i < 3; i++)
+				*(needle + i) = ' ';
+		}
+	}
+}
+
 static gboolean
 totem_pl_parser_ignore_from_mimetype (TotemPlParser *parser, const char *mimetype)
 {
@@ -1390,19 +1445,26 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 			|| g_file_has_uri_scheme (file, "rtsp") != FALSE
 			|| g_file_has_uri_scheme (file, "icy") != FALSE
 			|| g_file_has_uri_scheme (file, "pnm") != FALSE) {
-		DEBUG(file, g_print ("URL '%s' is MMS, RTSP, PNM or ICY, not a playlist\n", uri));
+		DEBUG(file, g_print ("URI '%s' is MMS, RTSP, PNM or ICY, not a playlist\n", uri));
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	}
 
 	/* Fix up itpc, see http://www.apple.com/itunes/store/podcaststechspecs.html,
-	 * as well as feed:// as used by Firefox 3 */
-	if (g_file_has_uri_scheme (file, "itpc") != FALSE || g_file_has_uri_scheme (file, "feed") != FALSE) {
-		DEBUG(file, g_print ("URL '%s' is getting special cased for ITPC/FEED parsing\n", uri));
+	 * feed:// as used by Firefox 3,
+	 * as well as zcast:// as used by ZENCast */
+	if (g_file_has_uri_scheme (file, "itpc") != FALSE
+	    || g_file_has_uri_scheme (file, "feed") != FALSE
+	    || g_file_has_uri_scheme (file, "zcast") != FALSE) {
+		DEBUG(file, g_print ("URI '%s' is getting special cased for ITPC/FEED/ZCAST parsing\n", uri));
 		return totem_pl_parser_add_itpc (parser, file, base_file, NULL);
 	}
+	if (g_file_has_uri_scheme (file, "zune") != FALSE) {
+		DEBUG(file, g_print ("URI '%s' is getting special cased for ZUNE parsing\n", uri));
+		return totem_pl_parser_add_zune (parser, file, base_file, NULL);
+	}
 	/* Try itms Podcast references, see itunes.py in PenguinTV */
 	if (totem_pl_parser_is_itms_feed (file) != FALSE) {
-	    	DEBUG(file, g_print ("URL '%s' is getting special cased for ITMS parsing\n", uri));
+	    	DEBUG(file, g_print ("URI '%s' is getting special cased for ITMS parsing\n", uri));
 	    	return totem_pl_parser_add_itms (parser, file, NULL, NULL);
 	}
 
@@ -1433,9 +1495,15 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 	DEBUG(file, g_print ("_get_mime_type_for_name for '%s' returned '%s'\n", uri, mimetype));
 	if (mimetype == NULL || strcmp (UNKNOWN_TYPE, mimetype) == 0
 	    || (g_file_is_native (file) && g_content_type_is_a (mimetype, "text/plain") != FALSE)) {
-		g_free (mimetype);
-		mimetype = my_g_file_info_get_mime_type_with_data (file, &data, parser);
-		DEBUG(file, g_print ("_get_mime_type_with_data for '%s' returned '%s'\n", uri, mimetype ? mimetype : "NULL"));
+	    	char *new_mimetype;
+		new_mimetype = my_g_file_info_get_mime_type_with_data (file, &data, parser);
+		if (new_mimetype) {
+			g_free (mimetype);
+			mimetype = new_mimetype;
+			DEBUG(file, g_print ("_get_mime_type_with_data for '%s' returned '%s'\n", uri, mimetype ? mimetype : "NULL"));
+		} else {
+			DEBUG(file, g_print ("_get_mime_type_with_data for '%s' returned NULL, ignoring\n", uri));
+		}
 	}
 
 	if (mimetype == NULL) {
@@ -1472,9 +1540,9 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 
 		for (i = 0; i < G_N_ELEMENTS(special_types); i++) {
 			if (strcmp (special_types[i].mimetype, mimetype) == 0) {
-				DEBUG(file, g_print ("URL '%s' is special type '%s'\n", uri, mimetype));
+				DEBUG(file, g_print ("URI '%s' is special type '%s'\n", uri, mimetype));
 				if (parser->priv->disable_unsafe != FALSE && special_types[i].unsafe != FALSE) {
-					DEBUG(file, g_print ("URL '%s' is unsafe so was ignored\n", uri));
+					DEBUG(file, g_print ("URI '%s' is unsafe so was ignored\n", uri));
 					g_free (mimetype);
 					g_free (data);
 					return TOTEM_PL_PARSER_RESULT_IGNORED;
@@ -1484,9 +1552,11 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 				else
 					base_file = g_object_ref (base_file);
 
+				DEBUG (file, g_print ("Using %s function for '%s'\n", special_types[i].mimetype, uri));
 				ret = (* special_types[i].func) (parser, file, base_file, data);
 
-				g_object_unref (base_file);
+				if (base_file != NULL)
+					g_object_unref (base_file);
 
 				found = TRUE;
 				break;
@@ -1495,7 +1565,7 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 
 		for (i = 0; i < G_N_ELEMENTS(dual_types) && found == FALSE; i++) {
 			if (strcmp (dual_types[i].mimetype, mimetype) == 0) {
-				DEBUG(file, g_print ("URL '%s' is dual type '%s'\n", uri, mimetype));
+				DEBUG(file, g_print ("URI '%s' is dual type '%s'\n", uri, mimetype));
 				if (data == NULL) {
 					g_free (mimetype);
 					mimetype = my_g_file_info_get_mime_type_with_data (file, &data, parser);
@@ -1513,7 +1583,8 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 
 				ret = (* dual_types[i].func) (parser, file, base_file ? base_file : file, data);
 
-				g_object_unref (base_file);
+				if (base_file != NULL)
+					g_object_unref (base_file);
 
 				found = TRUE;
 				break;
@@ -1530,7 +1601,7 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 		return ret;
 	}
 
-	if (totem_pl_parser_ignore_from_mimetype (parser, mimetype)) {
+	if (totem_pl_parser_ignore_from_mimetype (parser, mimetype) != FALSE) {
 		g_free (mimetype);
 		return TOTEM_PL_PARSER_RESULT_IGNORED;
 	}
@@ -1547,29 +1618,29 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 /**
  * totem_pl_parser_parse_with_base:
  * @parser: a #TotemPlParser
- * @url: the URL of the playlist to parse
+ * @uri: the URI of the playlist to parse
  * @base: the base path for relative filenames
- * @fallback: %TRUE if the parser should add the playlist URL to the
+ * @fallback: %TRUE if the parser should add the playlist URI to the
  * end of the playlist on parse failure
  *
- * Parses a playlist given by the absolute URL @url, using
+ * Parses a playlist given by the absolute URI @uri, using
  * @base to resolve relative paths where appropriate.
  *
  * Return value: a #TotemPlParserResult
  **/
 TotemPlParserResult
-totem_pl_parser_parse_with_base (TotemPlParser *parser, const char *url,
+totem_pl_parser_parse_with_base (TotemPlParser *parser, const char *uri,
 				 const char *base, gboolean fallback)
 {
 	GFile *file, *base_file;
 	TotemPlParserResult retval;
 
 	g_return_val_if_fail (TOTEM_IS_PL_PARSER (parser), TOTEM_PL_PARSER_RESULT_UNHANDLED);
-	g_return_val_if_fail (url != NULL, TOTEM_PL_PARSER_RESULT_UNHANDLED);
-	g_return_val_if_fail (strstr (url, "://") != NULL,
+	g_return_val_if_fail (uri != NULL, TOTEM_PL_PARSER_RESULT_UNHANDLED);
+	g_return_val_if_fail (strstr (uri, "://") != NULL,
 			TOTEM_PL_PARSER_RESULT_ERROR);
 
-	file = g_file_new_for_uri (url);
+	file = g_file_new_for_uri (uri);
 	base_file = NULL;
 
 	if (totem_pl_parser_scheme_is_ignored (parser, file) != FALSE) {
@@ -1593,19 +1664,19 @@ totem_pl_parser_parse_with_base (TotemPlParser *parser, const char *url,
 /**
  * totem_pl_parser_parse:
  * @parser: a #TotemPlParser
- * @url: the URL of the playlist to parse
- * @fallback: %TRUE if the parser should add the playlist URL to the
+ * @uri: the URI of the playlist to parse
+ * @fallback: %TRUE if the parser should add the playlist URI to the
  * end of the playlist on parse failure
  *
- * Parses a playlist given by the absolute URL @url.
+ * Parses a playlist given by the absolute URI @uri.
  *
  * Return value: a #TotemPlParserResult
  **/
 TotemPlParserResult
-totem_pl_parser_parse (TotemPlParser *parser, const char *url,
+totem_pl_parser_parse (TotemPlParser *parser, const char *uri,
 		       gboolean fallback)
 {
-	return totem_pl_parser_parse_with_base (parser, url, NULL, fallback);
+	return totem_pl_parser_parse_with_base (parser, uri, NULL, fallback);
 }
 
 /**
@@ -1614,16 +1685,21 @@ totem_pl_parser_parse (TotemPlParser *parser, const char *url,
  * @scheme: the scheme to ignore
  *
  * Adds a scheme to the list of schemes to ignore, so that
- * any URL using that scheme is ignored during playlist parsing.
+ * any URI using that scheme is ignored during playlist parsing.
  **/
 void
 totem_pl_parser_add_ignored_scheme (TotemPlParser *parser,
 		const char *scheme)
 {
+	char *s;
+
 	g_return_if_fail (TOTEM_IS_PL_PARSER (parser));
 
+	s = g_strdup (scheme);
+	if (s[strlen (s) - 1] == ':')
+		s[strlen (s) - 1] = '\0';
 	parser->priv->ignore_schemes = g_list_prepend
-		(parser->priv->ignore_schemes, g_strdup (scheme));
+		(parser->priv->ignore_schemes, s);
 }
 
 /**
@@ -1632,7 +1708,7 @@ totem_pl_parser_add_ignored_scheme (TotemPlParser *parser,
  * @mimetype: the mimetype to ignore
  *
  * Adds a mimetype to the list of mimetypes to ignore, so that
- * any URL of that mimetype is ignored during playlist parsing.
+ * any URI of that mimetype is ignored during playlist parsing.
  **/
 void
 totem_pl_parser_add_ignored_mimetype (TotemPlParser *parser,
@@ -1708,6 +1784,7 @@ totem_pl_parser_parse_duration (const char *duration, gboolean debug)
 	return -1;
 }
 
+
 /**
  * totem_pl_parser_parse_date:
  * @date_str: the date string to parse
@@ -1721,6 +1798,7 @@ totem_pl_parser_parse_duration (const char *duration, gboolean debug)
 guint64
 totem_pl_parser_parse_date (const char *date_str, gboolean debug)
 {
+#ifdef HAVE_CAMEL
 	GTimeVal val;
 
 	g_return_val_if_fail (date_str != NULL, -1);
@@ -1732,11 +1810,12 @@ totem_pl_parser_parse_date (const char *date_str, gboolean debug)
 		return val.tv_sec;
 	}
 	D(g_message ("Failed to parse duration '%s' using the ISO8601 parser", date_str));
-
 	/* Fall back to RFC 2822 date parsing */
 	return camel_header_decode_date (date_str, NULL);
+#else
+	WARN_NO_CAMEL;
+#endif /* HAVE_CAMEL */
 }
-
 #endif /* !TOTEM_PL_PARSER_MINI */
 
 static char *
@@ -1764,7 +1843,9 @@ totem_pl_parser_mime_type_from_data (gconstpointer data, int len)
 #endif
 
 	if (mime_type != NULL &&
-	    (strcmp (mime_type, "text/plain") == 0 || strcmp (mime_type, "application/octet-stream") == 0)) {
+	    (strcmp (mime_type, "text/plain") == 0 ||
+	     strcmp (mime_type, "application/octet-stream") == 0 ||
+	     strcmp (mime_type, "application/xml") == 0)) {
 		PlaylistIdenCallback func;
 		guint i;
 
@@ -1878,3 +1959,20 @@ totem_pl_parser_can_parse_from_filename (const char *filename, gboolean debug)
 	return retval;
 }
 
+#ifndef TOTEM_PL_PARSER_MINI
+GType
+totem_pl_parser_metadata_get_type (void)
+{
+	static volatile gsize g_define_type_id__volatile = 0;
+	if (g_once_init_enter (&g_define_type_id__volatile))
+	{ 
+		GType g_define_type_id = g_boxed_type_register_static (
+		    g_intern_static_string ("TotemPlParserMetadata"),
+		    (GBoxedCopyFunc) g_hash_table_ref,
+		    (GBoxedFreeFunc) g_hash_table_unref);
+		g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+	}
+	return g_define_type_id__volatile;
+}
+#endif /* !TOTEM_PL_PARSER_MINI */
+
